unit UnitFrmConfig;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, ExtCtrls;

type
  TFrmConfig = class(TForm)
    tvGroup: TTreeView;
    pcItems: TPageControl;
    tsGestures: TTabSheet;
    tsHelpPopup: TTabSheet;
    Panel1: TPanel;
    StaticText3: TStaticText;
    StaticText4: TStaticText;
    StaticText1: TStaticText;
    StaticText2: TStaticText;
    tbYPixel: TTrackBar;
    tbXPixel: TTrackBar;
    StaticText5: TStaticText;
    Panel2: TPanel;
    StaticText8: TStaticText;
    StaticText9: TStaticText;
    tbDeltaBias: TTrackBar;
    StaticText10: TStaticText;
    Panel3: TPanel;
    StaticText6: TStaticText;
    StaticText7: TStaticText;
    tbDistanceBias: TTrackBar;
    StaticText11: TStaticText;
    Panel4: TPanel;
    StaticText12: TStaticText;
    StaticText13: TStaticText;
    tbHelpPause: TTrackBar;
    StaticText14: TStaticText;
    cbEnableHelpPoup: TCheckBox;
    Label1: TLabel;
    Label2: TLabel;
    tsMisc: TTabSheet;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    tsScratchpad: TTabSheet;
    Panel5: TPanel;
    StaticText15: TStaticText;
    StaticText16: TStaticText;
    tbFadeTime: TTrackBar;
    StaticText17: TStaticText;
    cbEnableScratchpad: TCheckBox;
    Label6: TLabel;
    tsDiagonals: TTabSheet;
    cbEnableDiagonals: TCheckBox;
    Label7: TLabel;
    cbEnabledSuggestions: TCheckBox;
    Label8: TLabel;
    imgPixel: TImage;
    cbEnableCorrections: TCheckBox;
    Label9: TLabel;
    cbEnableDrawing: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure tvGroupClick(Sender: TObject);
    procedure tbYPixelChange(Sender: TObject);
    procedure tbXPixelChange(Sender: TObject);
  private
    { Private declarations }
    function GetDataFile : string;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure DrawPixels;
  public
    { Public declarations }
    procedure LoadSettings;
    procedure SaveSettings;

    procedure ShowPage(index : integer);
    
  end;

var
  FrmConfig: TFrmConfig;

implementation
uses IniFiles, UnitMisc, UnitFrmMouseMonitor, UnitFrmSysTrayMenu,
  UnitFrmScratchpad, UnitFrmGestures;
{$R *.dfm}

procedure TFrmConfig.CreateParams(var Params: TCreateParams);
begin
    inherited CreateParams(Params);
    with Params do begin
        ExStyle := ExStyle or WS_EX_APPWINDOW and not WS_EX_TOOLWINDOW ;
        WndParent := GetDesktopwindow;
    end;

end;

procedure TFrmConfig.DrawPixels;
begin
    with imgPixel.Canvas do begin
        brush.Color := clWhite;
        pen.Color := clBlack;
        Rectangle(imgpixel.Canvas.ClipRect);

        MoveTo(imgPixel.Width div 2, imgPixel.Height div 2 - (tbYPixel.Position div 2));
        LineTo(imgPixel.Width div 2, imgPixel.Height div 2 + (tbYPixel.Position div 2));
        MoveTo(imgPixel.Width div 2 - (tbXPixel.Position div 2), imgPixel.Height  div 2);
        LineTO(imgPixel.Width div 2 + (tbXPixel.Position div 2), imgPixel.Height  div 2);
    end;
end;

procedure TFrmConfig.FormClose(Sender: TObject; var Action: TCloseAction);
begin
    self.SaveSettings;
    self.LoadSettings;
end;

procedure TFrmConfig.FormCreate(Sender: TObject);
var i : integer;
begin
    self.LoadSettings;
    for i := 0 to pcItems.PageCount - 1 do begin
        pcItems.Pages[i].TabVisible := false;
    end;
    pcItems.Height := tvGroup.Height;
    pcItems.ActivePageIndex := 0;
end;

function TFrmConfig.GetDataFile: string;
begin
    result := UnitMisc.GetAppPath + 'defaults.ini'
end;

procedure TFrmConfig.LoadSettings;
var ini : TIniFile;
    function ReadINI(name : string; default : integer) : integer; overload;
    begin
        result := ini.ReadInteger('init',name,default);
        ini.WriteInteger('init',name,result);
    end;
    function ReadINI(name : string; default : double) : double; overload;
    begin
        result := ini.ReadFloat('init',name,default);
        ini.WriteFloat('init',name,result);
    end;
    function ReadINI(name : string; default : boolean) : boolean; overload;
    begin
        result := ini.ReadBool('init',name,default);
        ini.WriteBool('init',name,result);
    end;
    function LoadData( tb : TTrackBar) : integer; overload;
    begin
        result := ReadIni(tb.Name, tb.Position);
        tb.Position := result;
    end;
    function LoadDataFloat( tb : TTrackBar ) : single; overload;
    begin
        result := ReadIni(tb.Name, 1-(tb.Position/100));
        tb.Position := trunc((1-result)*100);
    end;
    function LoadData( cb : TCheckBox ) : boolean; overload;
    begin
        result := ReadIni(cb.name,cb.checked);
        cb.checked := result;
    end;
begin
    // NOTES:
    // This form MUST be created after all other forms.
    // This form must PUSH values for configuration, not pull

    // Create the INI file with defaults or read existing values
    // load the controls with those values
    ini := TiniFile.Create(self.GetDataFile);
    FrmMouseMonitor.prtimethreshold := ReadIni('timethreshold', 160);
    FrmMouseMonitor.prxThreshold := ReadIni('xThreshold',3);
    FrmMouseMonitor.pryThreshold := ReadIni('yThreshold',3);
    FrmMouseMonitor.prdirectionThreshold := ReadIni('directionThreshold',0);
    FrmMouseMonitor.prDeltaBias := LoadDataFloat(tbDeltaBias);
    FrmMouseMonitor.prdirectionBias := LoadDataFloat(tbDistanceBias);
    FrmMouseMonitor.prXPixelThreshold := LoadData(tbXPixel);
    FrmMouseMonitor.prYPixelThreshold := LoadData(tbYPixel);
    FrmMouseMonitor.prHelpPause := LoadData(tbHelpPause);
    FrmMouseMonitor.prEnableHelpPause := LoadData(cbEnableHelpPoup);
    FrmMouseMonitor.prEnableDiagonals := LoadData(cbEnableDiagonals);
    FrmMouseMonitor.prEnableDrawing := LoadData(cbEnableDrawing);

    FrmScratchPad.prFadeTime := LoadData(tbFadeTime);
    FrmScratchPad.prEnable := LoadData(cbEnableScratchpad);
    FrmGestures.prEnableSuggestions := LoadData(cbEnabledSuggestions);
    FrmGestures.prEnabledAutoCorrection := LoadData(cbEnableCorrections);
    FrmGestures.prExecutePause := ReadIni('ExecutePause', 100);
    myfree(ini);
end;

procedure TFrmConfig.SaveSettings;
var ini : TIniFile;
    procedure SaveData( wc : TWinControl);
    begin
        if wc is TTrackBar then begin
            if TTrackBar(wc).Hint = 'Decimal' then begin
                // a value of 20 will be written as 0.80 instead of (20 or 0.2)
                ini.WriteFloat('init', TTrackBar(wc).Name, 1 - (TTrackBar(wc).Position / 100));
            end else begin
                ini.WriteInteger('init', TTrackBar(wc).Name, TTrackBar(wc).Position);
            end;

        end else if wc is TEdit then begin
        end else if wc is TCheckBox then begin
            With TCheckBox(wc) do
                ini.WriteBool('init', name, checked );
        end;
    end;
    procedure TraverseControls(wc : TWinControl);
    var i : integer;
    begin
        for i := 0 to wc.ControlCount - 1 do begin
            if (wc.Controls[i] is TWinControl) then
                if (TWinControl(wc.Controls[i]).ControlCount > 0) then begin
                    TraverseControls(TWinControl(wc.Controls[i]));
                end else begin
                    SaveData(TWinControl(wc.Controls[i]));
                end;
        end;
    end;

begin
    ini := TIniFile.Create(self.GetDataFile);
    TraverseControls(self.pcItems);
    myfree(ini);
end;

procedure TFrmConfig.ShowPage(index: integer);
begin
    if index <> -1 then begin
        self.pcItems.ActivePageIndex := index;
        self.tvGroup.Items.Item[index].Selected := true;
    end;

     if ThreadAttach(GetForegroundWindow)  then begin
        self.show;
        Windows.SetForegroundWindow(Application.Handle);

        ThreadDetach;
    end;
end;

procedure TFrmConfig.tbXPixelChange(Sender: TObject);
begin
    self.DrawPixels;
end;

procedure TFrmConfig.tbYPixelChange(Sender: TObject);
begin
    self.DrawPixels;
end;

procedure TFrmConfig.tvGroupClick(Sender: TObject);
begin
    pcItems.ActivePageIndex := (tvGroup.Selected.AbsoluteIndex);
end;

end.
